var JackpotWindow;
(function () {
    var jackpotSettings = undefined;

    // Includes
    var Directory   = NovaEngine.System.Directory;
    var FileManager = NovaEngine.System.FileManager;
    var HtmlWindow  = NovaEngine.HtmlDocument.HtmlWindow;

    // Loading styles ad another scripts
    var jackpotWindowStylePath    = Directory.Combine("scripts/Game/Lib/Windows/Jackpot", "resources", "jackpot.css");
    var jackpotWindowTemplatePath = Directory.Combine("scripts/Game/Lib/Windows/Jackpot", "resources", "jackpot.html");
    var jackpotPlayerTemplatePath = Directory.Combine("scripts/Game/Lib/Windows/Jackpot", "resources", "player.html");
    var jackpotPlayerItemTemplatePath = Directory.Combine("scripts/Game/Lib/Windows/Jackpot", "resources", "im-item.html");
    var jackpotPlayerStartBoardItemTemplatePath = Directory.Combine("scripts/Game/Lib/Windows/Jackpot", "resources", "board-item.html");

    // Push Files in queue
    FileManager.Queue.AddFirst(jackpotWindowStylePath);
    FileManager.Queue.AddFirst(jackpotWindowTemplatePath);
    FileManager.Queue.AddFirst(jackpotPlayerTemplatePath);
    FileManager.Queue.AddFirst(jackpotPlayerItemTemplatePath);
    FileManager.Queue.AddFirst(jackpotPlayerStartBoardItemTemplatePath);

    function loadJackpotSettingsIdNeeded(callback) {
        if(!!jackpotSettings) {
            if(Function.prototype.isPrototypeOf(callback)) callback(jackpotSettings);
            return;
        }

        var wait = new Wpcl.Popup.Wait();
        Wpcl.Api.Methods.JackpotSettings({}, function (response) {
            if(response && response.status == 200)
                jackpotSettings = response.response;
            console.log(jackpotSettings.max_items);
            wait.Remove();
            if(Function.prototype.isPrototypeOf(callback)) callback(jackpotSettings);
        });
    }

    // Class for Jackpot window
    window.JackpotWindow = DeclareClass(function () {
        var that = this;

        var lowColor  = new Color(0, 255, 0, 1);
        var highColor = new Color(255, 0, 0, 1);

        var slowPushingMixTimeout = 1000,
            slowPushingMaxTimeout = 3000;

        var windowTemplate = FileManager.GetCachedFile(jackpotWindowTemplatePath).value;
        var playerTemplate = FileManager.GetCachedFile(jackpotPlayerTemplatePath).value;
        var playerItemTemplate = FileManager.GetCachedFile(jackpotPlayerItemTemplatePath).value;
        var playerStartBoardItemTemplate = FileManager.GetCachedFile(jackpotPlayerStartBoardItemTemplatePath).value;

        var layer = new HtmlLayer(windowTemplate);

        var players = [];
        var player = undefined;
        var played = false;
        var gameData = undefined;
        var maxWinnerRouletteUsers = 40;
        var winnerRouletteItems = {};

        var selectedItems = [];

        document.body.appendChild(layer.element);
        this.htmlWindow = new HtmlWindow(layer);
        this.Show = function () { that.htmlWindow.Show(); };
        this.Hide = function () { that.htmlWindow.Hide(); };

        // On Open Event
        this.htmlWindow.OnOpen(function () {
            played = false;
            startAction();
        });

        // On Close Event
        this.htmlWindow.OnClose(function () {
            updateAllIfPlayed();
        });

        layer.getBind("sum-chance-title") .innerHTML = LP("jackpot.chance");
        layer.getBind("jackpot-sum-title").innerHTML = LP("jackpot.jackpot");
        layer.getBind("items-push-title") .innerHTML = LP("jackpot.items");

        layer.getBind("sum-chance-value") .innerHTML = 0;
        layer.getBind("jackpot-sum-value").innerHTML = 0;

        layer.getBind("start.selected.items.title").innerHTML = LP("jackpot.selected.items");
        layer.getBind("go-enter").innerHTML = LP("jackpot.join");

        layer.getBind("btn-won-continue").innerHTML = LP("jackpot.continue");
        layer.getBind("btn-won-continue").OnTouchClick(function () {
            startAction();
        });

        layer.getBind("close").OnTouchClick(function () {
            Wpcl.windows.Game.Show();
        });

        layer.getBind("go-enter").OnTouchClick(function () {
            var bet = "";
            for(var item in selectedItems)
                bet += selectedItems[item].uwid + ",";
            bet = bet.replace(/,$/, '');
            
            Wpcl.Api.Methods.JackpotPlay({
                bet: bet
            }, function (response) {

                if(response.status == 200) {
                    played = true;

                    gameData = response.response;
                    runSlowItemsPushing();
                    showPlayUI();
                } else {

                    alert("Error: " + response.message);
                }
            });
        });

        this.__defineGetter__("players", function () { return players; });

        function showStartUI() {
            layer.getBind("start-ui").style.display = "block";
            layer.getBind("jackpot-info-im").removeAttribute("state");
            layer.getBind("jackpot-info").removeAttribute("state");
        }
        function hideStartUI() {
            layer.getBind("start-ui").style.display = "none";
        }

        function showPlayUI() {
            hideStartUI();
            layer.getBind("jackpot-info-im").setAttribute("state", "shown");
            layer.getBind("jackpot-info").setAttribute("state", "shown");
        }
        function hidePlayUI() {
            layer.getBind("jackpot-info-im").setAttribute("state", "hidden");
            layer.getBind("jackpot-info").setAttribute("state", "hidden");
        }

        function showWinnerRoulette() {
            layer.getBind('winner-roulette').style.display = "block";
        }
        function hideWinnerRoulette() {
            layer.getBind('winner-roulette').style.display = "none";
        }

        function showJackpotResult() {
            layer.getBind("jackpot-result").style.display = "block";
        }
        function hideJackpotResult() {
            layer.getBind("jackpot-result").style.display = "none";
        }

        function updateJackpotResultUI() {
            if(gameData.result == "lose") {
                layer.getBind("won-cash-title").innerHTML = LP("jackpot.lose");
                layer.getBind("won-cash-count").innerHTML = "";
            } else {
                layer.getBind("won-cash-title").innerHTML = LP("jackpot.win") + ":";
                layer.getBind("won-cash-count").innerHTML = that.jackpot / 100 + "$";
            }
        }
        
        function updateAllUI() {
            var itemsCount = 0;
            if(players.length > 0){
                if(players.length == 1) {
                    itemsCount = players[0].items.length;
                } else {
                    itemsCount = players.reduce(function (a, b) {
                        if(typeof a != "number") a = a.items.length;
                        return a + b.items.length;
                    });
                }
            }
            layer.getBind("sum-chance-value") .innerHTML = Math.floor(!!player ? player.chance : 0);
            layer.getBind("jackpot-sum-value").innerHTML = that.jackpot / 100;
            layer.getBind("items-push-value") .innerHTML = jackpotSettings.max_items + "/" + itemsCount;
        }

        this.__defineGetter__("itemsCount", function () {
            var itemsCount = 0;
            if(players.length > 0){
                if(players.length == 1) {
                    itemsCount = players[0].items.length;
                } else {
                    itemsCount = players.reduce(function (a, b) {
                        if(typeof a != "number") a = a.items.length;
                        return a + b.items.length;
                    });
                }
            }
            return itemsCount;
        });
        function addPlayerIm(google_id, items) {
            player = new Player(google_id, items, that);
            player.orientation = !!(players.length % 2) ? "left" : "right";
            players.push(player);

            layer.getBind("im-items").innerHTML = "";
            items.forEach(function (item) {
                var skin = Wpcl.Package.GameItems.package.weapons.skins[item.hash_id];
                if(!skin) return;

                var itemLayer = new HtmlLayer(playerItemTemplate);
                layer.getBind("im-items").appendChild(itemLayer.element);
                itemLayer.getBind("quality-color").style.backgroundColor = "#" + skin.quality_color;
                itemLayer.getBind("image").style.backgroundImage = "url(" +
                    NovaEngine.System.Directory.Combine(APP_ENV.entities_images_dir, skin.image)
                    + ")";
            });
        };
        function addPlayer(google_id, items) {
            var player = new Player(google_id, items, that);
            player.orientation = !!(players.length % 2) ? "left" : "right";
            players.push(player);
        };
        function redrawAll() {
            players.forEach(function (player) {
                player.redraw();
            });
            updateAllUI();
        };

        this.__defineGetter__("jackpot", function () {
            if(players.length <= 0) return 0;
            if(players.length == 1) return players[0].summaryPrice;
            return players.reduce(function (a, b) {
                if(typeof a !== "number") a = a.summaryPrice;
                return a + b.summaryPrice;
            });
        });

        function startAction() {
            hideWinnerRoulette();
            loadJackpotSettingsIdNeeded(function () {
                if(!jackpotSettings) { forceExit(); return; }

                layer.getBind("start.selected.items.max").innerHTML = jackpotSettings.max_items_per_player;
                layer.getBind("items-push-value") .innerHTML = jackpotSettings.max_items + "/" + 0;

                loadPlayerSkins();
            });
            clear();
            showStartUI();
        }
        
        function clear() {
            players.forEach(function (player) { player.remove(); });
            if(player) player.remove();

            players = [];
            selectedItems = [];
            player = undefined;
            gameData = undefined;
            winnerRouletteItems = {};
            layer.getBind("im-items").innerHTML = "";
            layer.getBind("start.items.list").innerHTML = "";

            checkJoinStatus();
            updateSelectedUI();
        };

        function Player(google_id, items, jackpot) {
            var that = this;
            var playerInfoLayer = new HtmlLayer(playerTemplate);

            var orientation = "left";
            var summaryPrice = 0;
            updateSummaryPrice();

            this.layer = playerInfoLayer;
            this.userId = google_id;
            this.name   = undefined;
            this.icon   = Directory.Combine("resources/images/user_unknown.webp");
            this.items  = items;

            this.redraw = function () {
                playerInfoLayer.getBind("icon-image").style.backgroundImage = "url(" + that.icon + ")";

                var chance = Math.floor(that.chance);

                playerInfoLayer.element.style.height = chance + "%";
                if(chance <= 3) {
                    playerInfoLayer.getBind("chance").style.display = "none";
                } else {
                    playerInfoLayer.getBind("chance").style.display = "block";
                    playerInfoLayer.getBind("chance").innerHTML = chance + "%";
                }
            };

            this.__defineGetter__("orientation", function () { return orientation; });
            this.__defineSetter__("orientation", function (o) {
                if(typeof o != "string") return
                o = o.toLowerCase();
                if(o != "left" && o != "right") return;

                orientation = o;
                playerInfoLayer.element.setAttribute("orientation", orientation);
            });

            this.__defineGetter__("summaryPrice", function () { return summaryPrice; });
            this.__defineGetter__("chance", function () {
                return 100 / jackpot.jackpot * summaryPrice;
            });

            this.remove = function () {
                if(playerInfoLayer.element.parentElement) playerInfoLayer.element.parentElement.removeChild(playerInfoLayer.element);
            };

            layer.getBind("players").appendChild(playerInfoLayer.element);

            var color = Color.prototype.lerp(lowColor, highColor, 100 / jackpotSettings.max_items * jackpot.itemsCount / 100);
            var colorStr = "rgba(" + color.r + ", " + color.g + ", " + color.b + ", 0.3)";
            playerInfoLayer.element.style.background = colorStr;
            playerInfoLayer.getBind("icon").style.borderColor = colorStr;

            this.redraw();
            loadPlayerInfo(this, function () {
                that.redraw();
            });

            function updateSummaryPrice() {
                if(!items.length) summaryPrice = 0;
                else if(items.length == 1) summaryPrice = items[0].price;
                else {
                    summaryPrice = items.reduce(function (a, b) {
                        if(typeof a !== "number") a = a.price;
                        return a + b.price;
                    });
                }
            }
        }
        function loadPlayerInfo(player, callback) {
            XHR("GET", "http://picasaweb.google.com/data/entry/api/user/" + player.userId + "?alt=json", null, function (response) {
                try {
                    response = JSON.parse(response);
                    if(!response.error && !!response.entry) {
                        var entry = response.entry;

                        if(entry["gphoto$nickname"] && entry["gphoto$nickname"]["$t"])
                            player.name = entry["gphoto$nickname"]["$t"];

                        if(entry["gphoto$thumbnail"] && entry["gphoto$thumbnail"]["$t"])
                            player.icon = entry["gphoto$thumbnail"]["$t"];

                        if(winnerRouletteItems[player.userId])
                            winnerRouletteItems[player.userId].style.backgroundImage = "url(" + player.icon + ")";

                        if(Function.prototype.isPrototypeOf(callback)) callback(true);
                    } else {
                        console.log("Error getting user info");
                        player.name = "UNKNOWN";
                        if(Function.prototype.isPrototypeOf(callback)) callback(false);
                    }
                } catch(e) {
                    console.log("Error getting user info");
                    player.name = "UNKNOWN";
                    if(Function.prototype.isPrototypeOf(callback)) callback(false);
                }
            });
        }

        function addItemToPlayerItemsBoard(item) {
            var skin = Wpcl.Package.GameItems.package.weapons.skins[item.hash_id];
            if(!skin) return;

            var itemLayer = new HtmlLayer(playerStartBoardItemTemplate);
            layer.getBind("start.items.list").appendChild(itemLayer.element);

            itemLayer.getBind("price").innerHTML = item.price / 100;
            itemLayer.getBind("quality-color").style.backgroundColor = "#" + skin.quality_color;
            itemLayer.getBind("content").style.backgroundImage = "url(" +
                Directory.Combine(APP_ENV.entities_images_dir, skin.image)
                + ")";

            itemLayer.element.OnTouchClick(function () {
                var index = selectedItems.indexOf(item);

                if(!!~index) {
                    selectedItems.splice(index, 1);
                    itemLayer.element.removeAttribute("selected");
                } else {
                    if(selectedItems.length >= jackpotSettings.max_items_per_player) return;

                    selectedItems.push(item);
                    itemLayer.element.setAttribute("selected", "");
                }

                updateSelectedUI();
                checkJoinStatus();
            });
        }

        function loadPlayerSkins() {
            var wait = new Wpcl.Popup.Wait();
            Wpcl.Api.Methods.SkinsPrices({}, function (response) {
                try {
                    wait.Remove();
                    if(response.status != 200) {
                        forceExit();
                        return;
                    }
                    var items = response.response;
                    items = items.sort(function(a, b) { return b.price - a.price});
                    items.forEach(function (item) {
                        addItemToPlayerItemsBoard(item);
                    });

                } catch(e) {
                    wait.Remove();
                    forceExit();
                }
            });
        }
        function checkJoinStatus() {
            layer.getBind("go-enter").style.display = (selectedItems.length > 0 ? "block" : "none");
        }
        function updateSelectedUI() {
            var selfJackpot = 0;

            for(var i in selectedItems)
                selfJackpot += selectedItems[i].price;

            layer.getBind("start.selected.items.count").innerHTML = selectedItems.length;
            layer.getBind("self.jackpot").innerHTML = selfJackpot / 100;
        }

        function runWinnerRoulette() {
            
            // Clear previous game results
            layer.getBind("winner-roulette-users-list").innerHTML = "";

            var playersChances = players.map(function (player) {
                return Math.ceil(player.chance);
            });

            var players_keys = Object.keys(players);
            var tmp_player = undefined;

            for(var i = 0; i < maxWinnerRouletteUsers; i++) {
                tmp_player = players[Math.weightedRandom(players_keys, playersChances)];
                insertUserUI(tmp_player)

                // Penultimate element
                if(i + 1 == maxWinnerRouletteUsers - 1) {
                    if(gameData.result == "lose" && tmp_player.userId == player.userId) {
                        insertUserUI(players[1]);
                    } else if(gameData.result == "win" && tmp_player.userId != player.userId) {
                        insertUserUI(player);
                    }
                }
            }
            setTimeout(function () {
                layer.getBind("winner-roulette-users-list").setAttribute("play", "");
                setTimeout(function () {
                    updateJackpotResultUI();
                    showJackpotResult ();
                }, 9000);
            }, 1000);

            function insertUserUI(player) {
                layer.getBind("winner-roulette-users-list").innerHTML += "<div class='user' style='background-image: url(" + player.icon + ")'></div>";
                winnerRouletteItems[player.userId] = layer.getBind("winner-roulette-users-list").lastChild;
                winnerRouletteItems[player.userId].setAttribute("player_id", player.userId);
            }
        }
        
        function runSlowItemsPushing() {
            var botIndex = 0;

            var playerItems = gameData.player.items.map(function (item_index) {
                return gameData.items[item_index];
            });
            addPlayerIm(gameData.player.google_id, playerItems);
            updateAllUI();

            pushNextPlayer();
            function pushNextPlayer() {
                if(!gameData.bots[botIndex]) {
                    setTimeout(function () {
                        IAds.showInterstitial();
                        hideJackpotResult ();
                        showWinnerRoulette();
                        runWinnerRoulette();
                        hidePlayUI();
                    }, 1000);
                    return;
                }

                setTimeout(function () {
                    var botItems = [];
                    console.log(gameData.bots[botIndex].items);
                    if(gameData.bots[botIndex].items) {
                        botItems = gameData.bots[botIndex].items.map(function (item_index) {
                        return gameData.items[item_index];
                    });
                    }
                    addPlayer(gameData.bots[botIndex].google_id, botItems);

                    botIndex++;
                    redrawAll();
                    pushNextPlayer();
                }, Math.randomRangeInt(slowPushingMixTimeout, slowPushingMaxTimeout));
            }
        }
        
        function forceExit() {
            updateAllIfPlayed();
            alert("Error");
            Wpcl.windows.Game.Show();
        }
        function updateAllIfPlayed() {
            if(played) {
                var wait = new Wpcl.Popup.Wait();
                Wpcl.Account.ForceSync(function () {
                    Wpcl.windows.Game.entities.rebuildAll();
                    Wpcl.windows.Inventory.rebuildAllSliders();
                    wait.Remove();
                });

                played = false;
            }
        }
        
        function preventDefaultEvent(e) {
            e.preventDefault();
            e.stopPropagation();
        }
    });
})();