(function() {
    var stylePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Marketplace", "Resources", "Marketplace.css");
    var templatePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Marketplace", "Resources", "Marketplace.html");
    var templateItemPath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Marketplace", "Resources", "MarketplaceItem.html");

    NovaEngine.System.FileManager.Queue.AddFirst(stylePath);
    NovaEngine.System.FileManager.Queue.AddFirst(templatePath);
    NovaEngine.System.FileManager.Queue.AddFirst(templateItemPath);

    Wpcl.InstantiateFunction("Windows", function Marketplace()
    {
        var that = this;

        var tmpl = NovaEngine.System.FileManager.GetCachedFile(templatePath).value;
        var layer = new HtmlLayer(tmpl);

        document.body.appendChild(layer.element);
        NovaEngine.HtmlDocument.HtmlWindow.call(this, layer);

        var package = Wpcl.Package.GameItems.package;
        var stickers = package.stickers;
        var marketLotTemplate = NovaEngine.System.FileManager.GetCachedFile(templateItemPath).value;
        var marketLotObject;
        var popupWait;

        var lotsTypes = {
            active: "active",
            player: "self"
        };

        var sortTypes = {
            asc: "asc",
            desc: "desc"
        };

        var qualityTypesArray = [
            "consumer",
            "industrial",
            "milspec",
            "restricted",
            "classified",
            "covert",
            "knive"
        ];

        var requestOptions = {};
        restoreOptions();

        var filterQualityRadioButtons = {};
        var filterWeaponRadioButtons = {};

        var el = {
            header: {
                title: layer.GetBindedElement('marketplace.title'),
                close: layer.GetBindedElement('marketplace.button.close')
            },
            body: layer.GetBindedElement('marketplace.body'),
            options: {
                keysCount: layer.GetBindedElement('marketplace.keys.count'),
                buttonActiveLots: layer.GetBindedElement('marketplace.button.active.lots'),
                buttonPlayerLots: layer.GetBindedElement('marketplace.button.player.lots'),
                buttonSort: layer.GetBindedElement('marketplace.sort.button'),
                bodyFilter: layer.GetBindedElement('marketplace.filter.line'),
                filterResult: layer.GetBindedElement('marketplace.filter.result'),
                filterSettingsButton: layer.GetBindedElement('marketplace.filter.settings.button')
            },
            settings: {
                wrapper: layer.GetBindedElement('marketplace.settings'),
                title: layer.GetBindedElement('marketplace.settings.header.title'),
                close: layer.GetBindedElement('marketplace.settings.header.close'),
                qualityFilterClear: layer.GetBindedElement('marketplace.settings.list.quality.clear'),
                qualityFilterTitle: layer.GetBindedElement('marketplace.settings.list.quality.title'),
                quailtyFilterContainer: layer.GetBindedElement('marketplace.settings.list.quality.container'),
                weaponFilterClear: layer.GetBindedElement('marketplace.settings.list.weapon.clear'),
                weaponFilterTitle: layer.GetBindedElement('marketplace.settings.list.weapon.title'),
                weaponFilterContainer: layer.GetBindedElement('marketplace.settings.list.weapon.container')
            }
        };

        constructor();
        el.header.title.innerHTML = LP("marketplace.title");
        el.settings.qualityFilterTitle.innerHTML = LP("common.select.quality");
        el.settings.weaponFilterTitle .innerHTML = LP("common.select.weapon");
        el.settings.title.innerHTML = LP("marketplace.filter.title");
        el.options.buttonActiveLots.innerHTML = LP("marketplace.button.lots.active");
        el.options.buttonPlayerLots.innerHTML = LP("marketplace.button.lots.player");

        that.OnOpen(function() {
            restoreOptions();
            clearBody();
            redrawUI();
            that.sendRequest();
        });

        this.restoreOptions = function() {
            restoreOptions();
        };

        function restoreOptions() {
            requestOptions = {
                offset: 0,
                quality: null,
                weaponName: null,
                lotsType: lotsTypes.active,
                sortType: sortTypes.desc,
                canLoad: true
            };
        }

        function redrawUI() {
            updateUIFilterResult();
            updateUIQualityFilter()
            updateUIRadioFilter();
            updateUIRadioLots();
            updateUIBody();
            updateUISort();
            updateUIWeaponFilter();
        }

        this.sendRequest = function() {
            var rObj = {};
            if(requestOptions.offset) rObj.offset = requestOptions.offset;

            popupWait = new Wpcl.Popup.Wait();
            switch(requestOptions.lotsType) {
                case lotsTypes.active:
                    if(requestOptions.hash_name) rObj.name = requestOptions.hash_name;
                    if(requestOptions.quality) rObj.quality = requestOptions.quality;
                    if(requestOptions.sortType) rObj.sort = requestOptions.sortType;
                    
                    Wpcl.Api.Methods.MarketplaceGetActiveList(rObj, onResponse);
                    break;
                case lotsTypes.player:
                    Wpcl.Api.Methods.MarketplaceGetPlayerList(rObj, onResponse);
                    break;
            }
        };

        function onResponse(data) {
            popupWait.Remove();

            if(data.status != 200) return alert("error");

            data.response.forEach(function(item) {
                createUILotObject(item);
            });

            if(data.response.length == 20) {
                requestOptions.offset += 20;
                createLoadMoreButton();
            } else {
                requestOptions.canLoad = false;
            }
        }

        function clearBody() {
            [].slice.call(el.body.children).forEach(function(child) {
                el.body.removeChild(child);
            });
        }

        function createLoadMoreButton() {
            var buttonNode = document.createElement("div");
            buttonNode.className = "market_load_next";
            buttonNode.innerHTML = LP("common.load.more");

            buttonNode.OnTouchClick(function() {
                that.sendRequest();
                el.body.removeChild(buttonNode);
            });

            el.body.appendChild(buttonNode);
        }

        function constructor() {
            initSettingsUI();
            initLotFabric();
            initEvents();
            updateUIFilterResult();
        }

        function initLotFabric() {
            marketLotObject = document.createElement('div');
            marketLotObject.innerHTML = marketLotTemplate;
            marketLotObject = marketLotObject.firstChild;
        }

        function createUILotObject(details) {
            var skin = package.weapons.skins[details.hash_id];
            if(!skin) return;

            var original = package.weapons.originals[skin.hash_name];
            if(!original) return;

            var qualityImageSrc = quality.colorToImage(skin.quality_color);
            var weaponImageSrc  = NovaEngine.System.Directory.Combine(APP_ENV.entities_images_dir, skin.image);

            var buttonImageSrc = "resources/images/"+(requestOptions.lotsType == lotsTypes.active ? "buy" : "close.bar")+".webp";

            var node = marketLotObject.cloneNode(true);
            var lotLayer = new HtmlLayer(node);

            var name_u = original.name;
            var skin_u = skin.name;

            lotLayer.GetBindedElement('market.lot.weapon.name').innerHTML = name_u;
            lotLayer.GetBindedElement('market.lot.weapon.quality').innerHTML = LP("case." + details.quality);
            lotLayer.GetBindedElement('market.lot.weapon.skin.name').innerHTML = skin_u;
            lotLayer.GetBindedElement('market.lot.overlay').style.backgroundImage = "url("+qualityImageSrc+")";
            lotLayer.GetBindedElement('market.lot.cover').style.backgroundImage = "url("+weaponImageSrc+")";
            lotLayer.GetBindedElement('market.lot.background').style.backgroundImage = "url("+qualityImageSrc+")";
            lotLayer.GetBindedElement('market.lot.buy').style.backgroundImage = "url("+buttonImageSrc+")";
            lotLayer.GetBindedElement('market.lot.price.value').innerHTML = Wpcl.Math.AttachSuffix(details.price / 100, 2);
            lotLayer.GetBindedElement('market.lot.price.currency').innerHTML = "$";

            var stickerSlot1 = lotLayer.GetBindedElement('market.lot.sticker1');
            var stickerSlot2 = lotLayer.GetBindedElement('market.lot.sticker2');
            var stickerSlot3 = lotLayer.GetBindedElement('market.lot.sticker3');

            DrawSticker(stickerSlot1, details.sticker1);
            DrawSticker(stickerSlot2, details.sticker2);
            DrawSticker(stickerSlot3, details.sticker3);

            lotLayer.GetBindedElement('market.lot.buy').OnTouchClick(function() {
                switch(requestOptions.lotsType) {
                    case lotsTypes.active:
                        if(Wpcl.Player.Cash.Get() < parseInt(details.price)) return alert("Enough keys");
                        if(confirm(  LP("combine.common.uwant1")
                                + name_u
                                + " "
                                + skin_u
                                + LP("combine.common.uwant2")
                                + details.price
                                + LP("combine.common.uwant3")))
                        {
                            Wpcl.Api.Methods.MarketplaceBuy({mid: details.mid}, function(data) {
                                if(data == null) return;
                                if(data.status === 200) {
                                    var res = data.response;
                                    el.body.removeChild(node);
                                    Wpcl.Player.weapons.insert(res.hash_name, {
                                        uwid        : res.uwid,
                                        hash_id     : res.hash_id,
                                        sticker1    : res.sticker1,
                                        sticker2    : res.sticker2,
                                        sticker3    : res.sticker3
                                    });

                                    var entity = Wpcl.windows.Game.entities[res.hash_name];
                                    if(!!entity) entity.rebuild(res.hash_name);

                                    Wpcl.Player.Cash.Set(parseInt(res.cash));
                                    Wpcl.Player.RecalculateStats();
                                    Wpcl.Account.syncHash = res.sync_hash;
                                } else {
                                    alert(LP("common.error.unknown"));
                                }
                            });
                        }
                        break;
                    case lotsTypes.player:
                        if(confirm(LP("marketplace.lot.remove"))) {
                            Wpcl.Api.Methods.MarketplaceRemove({mid: details.mid}, function(data) {

                                if(data == null) return;
                                if(data.status === 200) {
                                    el.body.removeChild(node);
                                    Wpcl.Player.weapons.insert(details.hash_name, {
                                        uwid        : details.uwid,
                                        hash_id     : details.hash_id,
                                        sticker1    : details.sticker1,
                                        sticker2    : details.sticker2,
                                        sticker3    : details.sticker3
                                    });

                                    
                                    var entity = Wpcl.windows.Game.entities[details.hash_name];
                                    if(!!entity) entity.rebuild(details.hash_name);

                                    Wpcl.Player.RecalculateStats();
                                    Wpcl.Account.syncHash = data.response.hash;
                                } else if(data.status === 512) {
                                    alert(LP("marketplace.lot.not.found"));
                                } else {
                                    alert(LP("common.error.unknown"));
                                }
                            });
                        }
                        break;
                }
            });

            el.body.appendChild(node);
            return node;
        }

        function DrawSticker(htmlSlot, stickerID)
        {

            if(!stickerID || !htmlSlot)
            {
                htmlSlot.style.display = "none";
                return;
            }

            var stickerData = stickers[stickerID];
            if(!stickerData)
            {
                htmlSlot.style.display = "none";
                return;
            }
            htmlSlot.style.backgroundImage = "url(" + NovaEngine.System.Directory.Combine(APP_ENV.entities_images_dir, stickerData.image) + ")";
            htmlSlot.style.display = "inline-block";
        }

        function removeUnderline(str) {
            return str.replace(/_/g, " ");
        }

        function initSettingsUI() {

            // ui quality filters
            quality.colors.forEach(function(quality) {
                if(quality == window.quality.colors.default) return;

                filterQualityRadioButtons[quality] = createRadioUI(quality, LP("quality." + quality), el.settings.quailtyFilterContainer, function() {
                    requestOptions.quality = quality;
                    updateUIQualityFilter();
                });
            });

            Object.keys(package.weapons.originals).forEach(function(key) {
                var original = package.weapons.originals[key];
                filterWeaponRadioButtons[original.hash_name] = createRadioUI(original.hash_name, original.name, el.settings.weaponFilterContainer, function() {
                    requestOptions.hash_name = original.hash_name;
                    updateUIWeaponFilter();
                });
            });

        }

        function initEvents() {
            el.header.close.OnTouchClick(eventCloseClick);
            el.options.buttonActiveLots.OnTouchClick(eventActiveLotsClick);
            el.options.buttonPlayerLots.OnTouchClick(eventPLayerLotsClick);
            el.options.buttonSort.OnTouchClick(eventSortChange);
            el.options.filterSettingsButton.OnTouchClick(eventSettingsClick);
            el.settings.qualityFilterClear.OnTouchClick(eventSettingsQualityClearClick);
            el.settings.weaponFilterClear.OnTouchClick(eventSettingsWeaponClearClick);

            //
            el.settings.close.OnTouchClick(eventSettingsClick);

            // ui keys event
            Wpcl.Player.Cash.OnChange(updateUIKeys);
            updateUIKeys(Wpcl.Player.Cash.Get());

            //
            updateUIRadioLots();
            updateUIBody();

            //
            updateUISort();
        }

        function updateUIBody() {
            switch(requestOptions.lotsType) {
                case lotsTypes.active:
                    el.body.classList.remove("market_size_player");
                    el.body.classList.add("market_size_active");
                    break;
                case lotsTypes.player:
                    el.body.classList.add("market_size_player");
                    el.body.classList.remove("market_size_active");
                    break;
            }
        }

        function updateUIRadioLots() {
            switch(requestOptions.lotsType) {
                case lotsTypes.active:
                    el.options.buttonActiveLots.classList.add("active");
                    el.options.buttonPlayerLots.classList.remove("active");
                    el.options.bodyFilter.style.display = "block";
                    break;
                case lotsTypes.player:
                    el.options.buttonActiveLots.classList.remove("active");
                    el.options.buttonPlayerLots.classList.add("active");
                    el.options.bodyFilter.style.display = "none";
                    break;
            }
        }

        function updateUISort() {
            switch(requestOptions.sortType) {
                case sortTypes.desc:
                    el.options.buttonSort.classList.add("arrow_down");
                    el.options.buttonSort.classList.remove("arrow_up");
                    break;
                case sortTypes.asc:
                    el.options.buttonSort.classList.remove("arrow_down");
                    el.options.buttonSort.classList.add("arrow_up");
                    break;
            }
        }

        function updateUIQualityFilter() {
            updateUIRadioFilter(filterQualityRadioButtons, requestOptions.quality);
            updateUIFilterResult();
        }

        function updateUIWeaponFilter() {
            updateUIRadioFilter(filterWeaponRadioButtons, requestOptions.hash_name);
            updateUIFilterResult();
        }

        function updateUIRadioFilter(arr, s) {
            for(var i in arr) {
                if(i === s) {
                    arr[i].node.classList.add('radio_active');
                } else {
                    arr[i].node.classList.remove('radio_active');
                }
            }
        }

        function updateUIKeys(keys) {
            el.options.keysCount.innerHTML = keys / 100 + "$";
        }

        function updateUIFilterResult() {
            var original = package.weapons.originals[requestOptions.hash_name];
            el.options.filterResult.innerHTML = (
                (original ? original.name : LP("common.all.weapons")))
                + "/"
                +  (requestOptions.quality ? LP("quality." + requestOptions.quality) : LP("common.all.quality"));
        }

        function eventCloseClick() {
            Wpcl.windows.Game.Show();
        }

        function eventSettingsClick() {
            switch(el.settings.wrapper.style.display) {
                case "block":
                    el.settings.wrapper.style.display = "none";
                    clearBody();
                    requestOptions.offset = 0;
                    that.sendRequest();
                    break;
                case "none":
                    el.settings.wrapper.style.display = "block";
                    break;
            }
        }

        function eventSortChange() {
            switch(requestOptions.sortType) {
                case sortTypes.asc:  requestOptions.sortType = sortTypes.desc; break;
                case sortTypes.desc: requestOptions.sortType = sortTypes.asc; break;
            }
            updateUISort();
            clearBody();
            requestOptions.offset = 0;
            that.sendRequest();
        }

        function eventSettingsQualityClearClick() {
            requestOptions.quality = null;
            updateUIQualityFilter();
        }

        function eventSettingsWeaponClearClick() {
            requestOptions.hash_name = null;
            updateUIWeaponFilter();
        }

        function eventActiveLotsClick() {
            if(requestOptions.lotsType == lotsTypes.active) return;
            clearBody();

            requestOptions.offset = 0;
            requestOptions.lotsType = lotsTypes.active;
            updateUIRadioLots();
            updateUIBody();
            that.sendRequest();
        }

        function eventPLayerLotsClick() {
            if(requestOptions.lotsType == lotsTypes.player) return;
            clearBody();

            requestOptions.offset = 0;
            requestOptions.lotsType = lotsTypes.player;
            updateUIRadioLots();
            updateUIBody();
            that.sendRequest();
        }

        function createRadioUI(id, name, parent, callback) {
            var radioWrapper = document.createElement('div');
            var radio = document.createElement('div');
            radioWrapper.className = "settings_radio";

            radio.innerHTML = name;
            radioWrapper.appendChild(radio);
            parent.appendChild(radioWrapper);

            radio.OnTouchClick(callback);

            return {
                setName: function(name) {
                    radio: name
                },
                node: radioWrapper
            };
        }
    });
    Wpcl.Windows.Marketplace.prototype = Object.create(NovaEngine.HtmlDocument.HtmlWindow.prototype);
    Wpcl.Windows.Marketplace.prototype.constructor = NovaEngine.HtmlDocument.HtmlWindow;

})();