(function() {
    var stylePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Shop", "Resources", "Shop.css");
    var styleOpencasePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Shop", "Resources", "Opencase.css");
    var templatePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Shop", "Resources", "Shop.html");
    var templateDonateItemPath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Shop", "Resources", "ShopDonateItem.html");
    var templateItemCasePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Shop", "Resources", "ShopItemCase.html");
    var templateOpenCasePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Shop", "Resources", "OpenCase.html");
    var templateShopItemPath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Shop", "Resources", "ShopItem.html");

    NovaEngine.System.FileManager.Queue.AddFirst(stylePath);
    NovaEngine.System.FileManager.Queue.AddFirst(styleOpencasePath);
    NovaEngine.System.FileManager.Queue.AddFirst(templatePath);
    NovaEngine.System.FileManager.Queue.AddFirst(templateDonateItemPath);
    NovaEngine.System.FileManager.Queue.AddFirst(templateItemCasePath);
    NovaEngine.System.FileManager.Queue.AddFirst(templateOpenCasePath);
    NovaEngine.System.FileManager.Queue.AddFirst(templateShopItemPath);

    Wpcl.InstantiateFunction("Windows", function Shop()
    {
        var Directory = NovaEngine.System.Directory;

        var tmpl = NovaEngine.System.FileManager.GetCachedFile(templatePath).value;
        var layer = new HtmlLayer(tmpl);

        document.body.appendChild(layer.element);
        NovaEngine.HtmlDocument.HtmlWindow.call(this, layer);

        var package = Wpcl.Package.GameItems.package;
        var that = this;

        var audioSource = new AudioSource();
        var audioClips = {
            "B0C3D9": new AudioClip("resources/audio/drop_common.ogg"),
            "5E98D9": new AudioClip("resources/audio/drop_uncommon.ogg"),
            "4B69FF": new AudioClip("resources/audio/drop_rare.ogg"),
            "8847FF": new AudioClip("resources/audio/drop_mythical.ogg"),
            "D32CE6": new AudioClip("resources/audio/drop_legendary.ogg"),
            "EB4B4B": new AudioClip("resources/audio/drop_legendary.ogg"),
            "E4AE39": new AudioClip("resources/audio/drop_legendary.ogg")
        };

        var stickersPage = 0,
            stickersOnPage = 8,
            stickersPageButtons = [],
            pageStickers = [];

        var tabs = [];

        var opencaseLayer;
        var blurNode;
        var opencaseQImageNode,
            opencaseTitleNode,
            opencaseImageNode,
            opencasePriceNode,
            opencasestattrakNode;

        var donateItemTemplate = NovaEngine.System.FileManager.GetCachedFile(templateDonateItemPath).value;
        var DONATE_STATE = {
            unknown: "UNK",
            available: "AVA",
            unavailable: "UAVA",
            requesting: "REQ"
        };
        var donateState = DONATE_STATE.unknown;

        var tabStickersContainer,
            tabDonateContainer,
            tabCasesContainer;

        var _casesAlreadyLoaded = false;

        that.OnOpen(function() {
            requestCasesInfo();
        });

        that.OnClose(function() {});

        function updateUIKeys(keys) {
            layer.GetBindedElement('options.keys.count').innerHTML = keys / 100 + "$";
        }
        function drawDonateItem(productID, rewardType, rewardID, rewardCount, discount) {
            var donateItemLayer =  new HtmlLayer(donateItemTemplate);
            var node = donateItemLayer.element;
            
            if(discount) node.setAttribute("discount", discount > 500 ? "mega" : ("+ " + discount + " %"));
            donateItemLayer.GetBindedElement("reward.image").style.backgroundImage = "url(resources/images/donate/" + rewardType + rewardID + ".webp)";
            donateItemLayer.GetBindedElement("reward.type").innerHTML = LP("donate.reward.type." + rewardType);
            donateItemLayer.GetBindedElement("reward.count").innerHTML = Wpcl.Math.AttachSuffix(rewardCount);
            donateItemLayer.GetBindedElement("reward.buy").innerHTML = LP("donate.reward.buy");

            donateItemLayer.GetBindedElement("reward.buy").OnTouchClick(function() {
                if(!Wpcl.Payments.GoogleIAP.ready)
                {
                    Wpcl.Payments.GoogleIAP.Init();
                    alert('Please try again later');
                    return;
                }

                Wpcl.Payments.GoogleIAP.PurchaseProduct(productID);
            });

            layer.getBind("money-keys-container").appendChild(node);
        }
        function donateTabOpenEvent() {
            switch(donateState) {
                case DONATE_STATE.unknown:
                    donateState = DONATE_STATE.requesting;
                    Wpcl.Api.Methods.DonateGetPurchaseList({}, function(data) {
                        var tmp;
                        var tmpMatch;

                        if(data.status == 200) {
                            layer.getBind("money-keys-container").innerHTML = "";

                            for(var i in data.response)
                            {
                                tmp = data.response;
                                for(var o in tmp[i]) {
                                    tmpMatch = tmp[i][o].productID.match("cash") || tmp[i][o].productID.match("credits");

                                    if(!tmpMatch) continue;

                                    drawDonateItem(tmp[i][o].productID,
                                        tmpMatch[0],
                                        tmp[i][o].productID.replace(/[^-0-9]/gim,''),
                                        tmp[i][o].reward,
                                        tmp[i][o].discount);
                                    Wpcl.Payments.GoogleIAP.AddProductId(tmp[i][o].productID);
                                }
                            }

                            Wpcl.Payments.GoogleIAP.Init();
                            donateState = DONATE_STATE.available;
                        } else {
                            donateState = DONATE_STATE.unavailable;
                        }
                    });
                    return;
                case DONATE_STATE.available:
                case DONATE_STATE.unavailable:
                case DONATE_STATE.requesting:
                default: return;
            }
        }


        function requestCasesInfo() {
            if(_casesAlreadyLoaded) return;
            var p_wait = new Wpcl.Popup.Wait();
            Wpcl.Api.Methods.GetCasesList({}, function(data) {
                if(data.status == 200) {
                    p_wait.Remove();
                    drawCases(data.response, layer.getBind("cases-container"));
                    _casesAlreadyLoaded = true;
                }
            });
        }

        function Init() {
            layer.GetBindedElement("header.close").OnTouchClick(function() {
                Wpcl.windows.Game.Show();
            });

            InitOpencase();
            InitTabs();
            initStickersPageSystem();
            drawStickersPage();

            layer.GetBindedElement('header.title').innerHTML = LP("shop.title");
            Wpcl.Player.Cash.OnChange(updateUIKeys);
            updateUIKeys(Wpcl.Player.Cash.Get());
        }
        function InitOpencase() {
            blurNode = layer.element.firstElementChild;

            var openCaseTmpl = NovaEngine.System.FileManager.GetCachedFile(templateOpenCasePath).value;
            opencaseLayer = new HtmlLayer(openCaseTmpl);

            layer.element.appendChild(opencaseLayer.element);
            opencaseLayer.Hide();

            opencaseQImageNode = opencaseLayer.getBind("light.quality.image");
            opencaseTitleNode  = opencaseLayer.getBind("title");
            opencaseImageNode  = opencaseLayer.getBind("image");
            opencasePriceNode  = opencaseLayer.getBind("price");
            opencasestattrakNode = opencaseLayer.getBind("stattrak");

            opencaseLayer.getBind("continue").innerHTML = LP('common.continue');

            opencaseLayer.getBind("continue").OnTouchClick(function() {
                that.hideOpencase();
            });
        }

        function InitTabs() {
            var tabButtonStickers = layer.GetBindedElement('options.stickers.button'),
                tabButtonDonate = layer.GetBindedElement('options.donate.button'),
                tabButtonCases = layer.GetBindedElement('options.cases.button');

            tabStickersContainer = layer.GetBindedElement('nav-content-stickers');
            tabDonateContainer = layer.GetBindedElement('nav-content-money-keys');
            tabCasesContainer = layer.GetBindedElement('nav-content-cases');

            tabButtonStickers.innerHTML = LP("shop.nav.stickers");
            tabButtonDonate.innerHTML = LP("shop.nav.donate");
            tabButtonCases.innerHTML = LP("shop.nav.cases");

            that.addTab("stickers",
                tabButtonStickers,
                tabStickersContainer,
                tabs, false, function() {
                    openStickersPage(0);
                });

            that.addTab("money-keys",
                tabButtonDonate,
                tabDonateContainer,
                tabs, false, donateTabOpenEvent);

            that.addTab("cases",
                tabButtonCases,
                tabCasesContainer,
                tabs, true);
        };

        var itemStickertemp = NovaEngine.System.FileManager.GetCachedFile(templateShopItemPath).value;


        function CreateStickerItem(sticker)
        {
            if(!sticker) return;

            var itemLayer = new HtmlLayer(itemStickertemp);
            //itemLayer.GetBindedElement('shop.item.title').innerHTML = sticker.name;
            itemLayer.GetBindedElement('shop.item.cover').style.backgroundImage =
                "url(" +NovaEngine.System.Directory.Combine(APP_ENV.entities_images_dir, sticker.image) + ")";

            //itemLayer.GetBindedElement('shop.item.button.wrap').classList.add("weapon_quality_" + sticker.quality_color + "_bg");
            itemLayer.GetBindedElement('shop.item.click.value').innerHTML  = Wpcl.Math.AttachSuffix(sticker.click,  1);
            itemLayer.GetBindedElement('shop.item.income.value').innerHTML = Wpcl.Math.AttachSuffix(sticker.income, 1);
            itemLayer.GetBindedElement('price').innerHTML = Wpcl.Math.AttachSuffix(sticker.price / 100, 1) + "$";
            itemLayer.GetBindedElement('shop.item.click.name').innerHTML = LP("common.click");
            itemLayer.GetBindedElement('shop.item.income.name').innerHTML = LP("common.income");
            //itemLayer.GetBindedElement('shop.item.button.buy').innerHTML = LP("shop.button.buy");
            itemLayer.element.OnTouchClick(function(){
                that.common.stickerSelect.state = true;
                that.common.stickerSelect.data  = sticker;

                var p_wait = new Wpcl.Popup.Wait();

                Wpcl.windows.Game.entities.rebuildAll();
                Wpcl.windows.Inventory.Show();

                p_wait.Remove();
            });

            layer.GetBindedElement('stickers-container').appendChild(itemLayer.element);
        }

        function clearStickersContainer()
        {
            layer.GetBindedElement('stickers-container').innerHTML = "";
        }

        function initStickersPageSystem()
        {
            var stickersKeys = Object.keys(package.stickers);
            var pageIndexes = stickersKeys.length / stickersOnPage;

            for(var i = 0; i < pageIndexes; i++)
            {
                createStickersPageScrollButton(i);
            }
            normalizePageButtons();
        }

        function createStickersPageScrollButton(index)
        {
            var button = document.createElement("div");
            button.className = "shop-page-scroll-button";
            button.innerHTML = index + 1;

            button.OnTouchClick(function () {
                openStickersPage(index);
            });

            stickersPageButtons.push(button);
            layer.getBind('stickers-page-scroll').appendChild(button);
        }

        function openStickersPage(index)
        {
            stickersPage = index;

            var wait = new Wpcl.Popup.Wait();

            var stickersKeys = Object.keys(package.stickers);
            var offset = stickersPage * stickersOnPage;
            pageStickers = [];

            for(var i = offset; i < offset + stickersOnPage; i++)
                pageStickers.push(package.stickers[stickersKeys[i]]);

            var stickersQuaery = [];

            pageStickers.forEach(function (sticker) {
                if(!sticker) return;
                stickersQuaery.push(sticker.hash_id);
            });
            stickersQuaery = JSON.stringify(stickersQuaery);
            Wpcl.Api.Methods.StickersPrice({stickers: stickersQuaery}, function(response) {
                wait.Remove();

                if(!response || !response.response)
                {
                    alert("Error");
                    return;
                }

                var response = response.response;

                for(var i in response)
                {
                    if(!package.stickers[i]) continue;
                    package.stickers[i].price = response[i];
                }

                normalizePageButtons();
                drawStickersPage();
            });

        }

        function normalizePageButtons()
        {
            var buttons = stickersPageButtons;
            for(var i in buttons)
            {
                if(i == stickersPage)
                {
                    buttons[i].setAttribute("active", "");
                }
                else
                {
                    buttons[i].removeAttribute("active");
                }
            }
        }

        function drawStickersPage()
        {
            clearStickersContainer();
            pageStickers.forEach(function (sticker) {
                CreateStickerItem(sticker);
            });
        }

        function drawCases(list, parent) {
            var itemtemp = NovaEngine.System.FileManager.GetCachedFile(templateItemCasePath).value;
            var cases = [];

            // Фильтрация доступных кейсов кейсов
            for(var i in list) {
                if(package.cases[list[i].hash_name]) {
                    cases.push(list[i]);
                }
            }

            // Отисовка кейсов
            for(var o in cases) {
                if(o == 0 && cases.length % 2 > 0) {
                    drawCaseByInfo(package.cases[cases[o].hash_name], cases[o].price, true); // Широкий формат
                } else {
                    drawCaseByInfo(package.cases[cases[o].hash_name], cases[o].price);
                }
            }
            
            function drawCaseByInfo(caseDetails, casePrice, wideMode){
                var caseLayer = new HtmlLayer(itemtemp);

                // Применение широкого формата
                if(wideMode === true) {
                    caseLayer.element.setAttribute("wide", "");
                }

                caseLayer.getBind('image').style.backgroundImage = "url(resources/cases_src/" + caseDetails.image + ")";
                caseLayer.getBind('title').innerHTML = caseDetails.name;
                caseLayer.getBind('price').innerHTML = casePrice / 100 + "$";
                caseLayer.element.OnTouchClick(function() {
                    // Нет ключей, выходим
                    if(Wpcl.Player.Cash.Get() < casePrice) return;

                    var p_wait = new Wpcl.Popup.Wait();

                    // Отправляем запрос серверу на открытие кейса
                    Wpcl.Api.Methods.OpenCase({case_id: caseDetails.hash_name}, function(response) {
                        p_wait.Remove();

                        if(response.status != 200) return console.log(response);

                        // Ошибка в ответе
                        // x64 - недостаточно ключей
                        if(response.status === 64) return alert(LP("common.not.enough.keys"));

                        // x0 - неизвестная ошибка или ошибка в парсинге результата
                        if(response.status === 0)  return alert(LP("common.not.connection"));

                        // Результат запроса
                        var response = response.response;

                        // Скин полученого оружия
                        var skin = package.weapons.skins[response.hash_id];
                        if(!skin) return alert('error');

                        // Оригинальное оружие
                        var original = package.weapons.originals[response.hash_name];
                        if(!original) return alert('error')

                        // Открываем окно презентации оружия

                        that.showOpencase(
                            original.name + " | " + skin.name,
                            skin.quality_color,
                            Directory.Combine(APP_ENV.entities_images_dir, skin.image),
                            response.price,
                            response.stattrak,
                            response.souvenir
                        );
                        Wpcl.Account.syncHash = response.sync_hash;

                        Wpcl.Player.weapons.insert(response.hash_name, {
                            uwid        : response.uwid,
                            hash_id     : response.hash_id,
                            sticker1    : response.sticker1,
                            sticker2    : response.sticker2,
                            sticker3    : response.sticker3
                        });
                        Wpcl.windows.Game.entities.rebuild(skin.hash_name);

                        audioSource.playOneShot(audioClips[skin.quality_color]);
                        Wpcl.Player.Cash.Set(response.cash);
                    });
                });
                parent.appendChild(caseLayer.element);

                Wpcl.Player.Cash.OnChange(updateUI);
                updateUI(Wpcl.Player.Cash.Get());

                function updateUI(keys) {
                    if(keys < casePrice) {
                        caseLayer.element.classList.add('non-active');
                    } else {
                        caseLayer.element.classList.remove('non-active');
                    }
                };
            };
        };

        //
        function Tab(id, button, content, tabs, showCallback, hideCallback) {
            var that = this;

            this.id = id;
            this.button  = button;
            this.content = content;

            this.hide = function() {
                this.button.classList.remove('active');
                this.content.style.display = "none";

                if(Function.prototype.isPrototypeOf(hideCallback))
                    hideCallback();
            };

            this.show = function() {
                this.button.classList.add('active');
                this.content.style.display = "block";

                if(Function.prototype.isPrototypeOf(showCallback))
                    showCallback();
            };

            button.OnTouchClick(function() {
                that.show();
                hideOtherTabs();
            });

            function hideOtherTabs() {
                tabs.forEach(function(tab) {
                    if(tab.id != id) {
                        tab.hide();
                    }
                });
            }
        }

        this.common = {};
        this.common.stickerSelect = {};
        this.common.stickerSelect.state = false;
        this.common.stickerSelect.data  = {};

        this.showOpencase = function(title, quality, image, price, stattrak, souvenir) {
            opencaseLayer.Show();
            layer.element.removeChild(opencaseLayer.element);
            layer.element.insertBefore(opencaseLayer.element, blurNode);

            opencaseQImageNode.src = "resources/light/"+quality+".webp";
            opencaseTitleNode.innerHTML = title;
            opencasePriceNode.innerHTML = price / 100 + "$";
            opencaseImageNode.src = image;
            blurNode.classList.add('blur');

            opencasestattrakNode.style.display = stattrak ? "block" : "none";
            if(souvenir) opencasePriceNode.innerHTML += "<i souvenir>souvenir</i>";
        };
        this.hideOpencase = function() {
            opencaseLayer.Hide();
            blurNode.classList.remove('blur');
        };

        this.addTab = function(id, button, content, tabs, showed, showCallback, hideCallback) {
            var tab = new Tab(id, button, content, tabs, showCallback, hideCallback);
            showed ? tab.show() : tab.hide();

            tabs.push(tab);
        };

        Init();
    });
    Wpcl.Windows.Shop.prototype = Object.create(NovaEngine.HtmlDocument.HtmlWindow.prototype);
    Wpcl.Windows.Shop.prototype.constructor = NovaEngine.HtmlDocument.HtmlWindow;
})();