(function() {
    var stylePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Special", "Resources", "Special.css");
    var templatePath = NovaEngine.System.Directory.Combine("scripts/Game/Lib/Windows/Special", "Resources", "Special.html");

    NovaEngine.System.FileManager.Queue.AddFirst(stylePath);
    NovaEngine.System.FileManager.Queue.AddFirst(templatePath);

    Wpcl.InstantiateFunction("Windows", function Special()
    {
        var that = this;
        var tmpl = NovaEngine.System.FileManager.GetCachedFile(templatePath).value;
        var layer = new HtmlLayer(tmpl);

        document.body.appendChild(layer.element);
        NovaEngine.HtmlDocument.HtmlWindow.call(this, layer);

        var resetProgressSecretKey = "crypto_12CAs4325fDS";

        var t_d,
            t_h,
            t_m,
            t_s;

        var LOCK_STATUS = {
            unknown: 0,
            locked: 1,
            ready: 2,
            requesting: 3
        };

        var converter = {
            data: {
                profit: 0,
                next_lock: 0,
                id: 0
            },
            calcMoneyDecCount: function() {
                return 50000 * Math.pow(1.5, this.data.id);
            },
            lock: {
                status: LOCK_STATUS.unknown,
                term: 0,
                clientLTime: 0,
                serverLTime: 0,
                startWaitInterval: function() {
                    var interval = setInterval(function() {
                        if(converter.lock.term <= 0) {
                            clearInterval(interval);
                            converter.lock.status = LOCK_STATUS.unknown;
                            converterChecker();
                        }

                        converter.lock.term--;
                        updateConverterTimerUI();
                    }, 1010 /* 1010 для исключения повторных запросов */);
                },
                requestStatus: function() {
                    this.status = LOCK_STATUS.requesting;
                    updateConverterUI();

                    Wpcl.Api.Methods.ConvertStatus({}, function(data) {
                        if(data && data.response)
                        {
                            switch(data.response.status) {
                                case "LOCKED":
                                    converter.lock.status = LOCK_STATUS.locked;
                                    converter.lock.clientLTime = Date.now() / 1000;
                                    converter.lock.serverLTime = data.response.blocked_until;
                                    converter.lock.term = data.response.to_unlock;
                                    converter.lock.startWaitInterval();

                                    updateConverterTimerUI();
                                    updateConverterUI();
                                    break;
                                case "READY":
                                    converter.lock.status = LOCK_STATUS.ready;
                                    converter.data.profit = data.response.profit;
                                    converter.data.next_lock = data.response.lock_for;
                                    converter.data.id = data.response.id;
                                    updateConverterUI();
                                    break;
                            }
                        }
                        else
                        {
                            converter.lock.status = LOCK_STATUS.unknown;
                            updateConverterUI();
                        }
                    });
                }
            },
            requestConvert: function() {
                var dec = this.calcMoneyDecCount();

                if(this.lock.status != LOCK_STATUS.ready)
                    return alert(LP('common.error.unknown'));

                if(Wpcl.Player.Credits.Get() < dec)
                    return alert(LP('common.not.enough.money'));

                this.lock.status = LOCK_STATUS.requesting;
                updateConverterUI();

                Wpcl.Api.Methods.Convert({}, function(data) {
                    console.log(data);
                    if(data.status == 200) {
                        converter.lock.status = LOCK_STATUS.locked;
                        converter.lock.clientLTime = Date.now() / 1000;
                        converter.lock.serverLTime = data.response.blocked_until;
                        converter.lock.term = data.response.lock_for;
                        converter.lock.startWaitInterval();

                        updateConverterTimerUI();
                        updateConverterUI();

                        Wpcl.Player.Cash.Set(data.response.cash);
                        Wpcl.Player.Cash.ForceSave();

                        Wpcl.Player.Credits.Dec(dec);
                        Wpcl.Player.Credits.ForceSave();

                        IAds.showInterstitial();
                    } else {
                        converter.requestStatus();
                    }
                    updateConverterUI();
                });
            },
            ui: {}
        };
        OnStart();

        this.OnOpen(function() {
            converterChecker();
            updateWKIDUI();
        });

        function OnStart() {
            layer.getBind('special.title').innerHTML = LP('special.title');
            layer.getBind('special.button.close').OnTouchClick(function() {
                Wpcl.windows.Game.Show();
            });

            setupResetUnit();
            setupVoteUnit();
            setupOfferUnit();
            setupConverter();
            setupSocialUnit();
            setupLanguageUnit();
            //setupPolandFriendAdUnit();
            setupAccountActionsUnit();
            updateWKIDUI();
        };

        function converterChecker() {
            if(converter.lock.status == LOCK_STATUS.unknown)
                converter.lock.requestStatus();
        }

        function updateConverterTimerUI() {
            t_d = converter.lock.term;
            t_h = Math.floor(t_d / 3600 % 24);
            t_m = Math.floor(t_d / 60 % 60);
            t_s =  t_d % 60;

            converter.ui.mTime.innerHTML = formatTime(t_h, t_m, t_s);
        }

        function formatTime(h, m, s) {
            return formatTimeNumber(h) + ":" + formatTimeNumber(m) + ":" + formatTimeNumber(s);
        }

        function formatTimeNumber(v) {
            return (v.toString().length == 1 ? ("0" + v) : v);
        }

        function updateConverterUI() {
            switch(converter.lock.status) {
                case LOCK_STATUS.locked:
                    converter.ui.body.classList.add("special_converter_not_avaliable");
                    converter.ui.body.classList.remove("special_converter_wait");
                    converter.ui.body.classList.remove("special_converter_error");
                    break;
                case LOCK_STATUS.ready:
                    converter.ui.body.classList.remove("special_converter_not_avaliable");
                    converter.ui.body.classList.remove("special_converter_wait");
                    converter.ui.body.classList.remove("special_converter_error");
                    converter.ui.sFrom.innerHTML = Wpcl.Math.AttachSuffix(converter.calcMoneyDecCount());
                    converter.ui.sTo.innerHTML   = converter.data.profit;
                    break;
                case LOCK_STATUS.unknown:
                    converter.ui.body.classList.remove("special_converter_wait");
                    converter.ui.body.classList.add("special_converter_error");
                    break;
                case LOCK_STATUS.requesting:
                    converter.ui.body.classList.add("special_converter_wait");
                    converter.ui.body.classList.remove("special_converter_error");
                    break;
            }
        }

        function setupConverter() {

            converter.ui.mDetails = layer.getBind('special.converter.message.details');
            converter.ui.message  = layer.getBind('special.converter.message');
            converter.ui.scheme   = layer.getBind('special.converter.scheme');
            converter.ui.action   = layer.getBind('special.converter.action');
            converter.ui.mTime    = layer.getBind('special.converter.message.time');
            converter.ui.sFrom    = layer.getBind('special.converter.scheme.from');
            converter.ui.title    = layer.getBind('special.converter.title');
            converter.ui.body     = layer.getBind('special.converter');
            converter.ui.sTo      = layer.getBind('special.converter.to');

            converter.ui.title.innerHTML = LP('special.converter.title');
            converter.ui.mDetails.innerHTML = LP('special.converter.message.unavaliable');

            converter.ui.action.OnTouchClick(function() {
                converter.requestConvert();
            });

            updateConverterUI();
        }

        function updateWKIDUI() {
            layer.getBind('special.wkid.code').innerHTML = Wpcl.Account.uid;
        }

        function setupAccountActionsUnit() {
            //var aSignOut = layer.getBindedElement('special.account.sign.out');
            //var aClearCache = layer.getBindedElement('special.account.clear.cache');

            //aSignOut.innerHTML = "sign out";
            //aClearCache.innerHTML = "clear cache";

            /*aSignOut.onTouchClick(function() {
             if(confirm("Are you sure you want to sign out; local progress will be lost")) {
             core.modules.update.stop();
             localStorage.clear();
             window.player.inventory.clearProgress();
             window.account.ui.dialogSignIn();
             core.modules.layer.showLayerById('account', true);
             }
             });*/
        }

        function setupResetUnit() {
            layer.getBind('special.reset.progress').OnTouchClick(function() {
                var w_win = new Wpcl.Popup.Wait();
                var e_progress = generateProgressString();

                Wpcl.Api.Methods.ResetProgress({action: "calculate", progress: e_progress}, function(data) {
                    console.log(e_progress);
                    if(data.status == 200) {
                        if(confirm("reset progress? you get " + (data.response / 100) + "$")) {
                            Wpcl.Api.Methods.ResetProgress({action: "reset", progress: e_progress}, function(data) {
                                if(data.status == 200) {
                                    Wpcl.Player.Cash.Inc(data.response);
                                    Wpcl.Player.Credits.Reset();
                                    Wpcl.Rank.Reset();
                                    Wpcl.windows.Game.entities.clearProgress();


                                    IAds.showInterstitial();
                                } else {
                                    alert(data.error);
                                }
                                w_win.Remove();
                            });
                        } else {
                            w_win.Remove();
                        }
                    } else {
                        alert(data.error);
                        w_win.Remove();
                    }
                });
            });
        }

        function generateProgressString()
        {
            return GibberishAES.enc((Wpcl.Player.Credits.Get() + Wpcl.Rank.Experience), resetProgressSecretKey);
        }

        function setupVoteUnit() {
            layer.getBind('special.like').OnTouchClick(function() {
                cordova.InAppBrowser.open('https://play.google.com/store/apps/details?id=com.valoriros.weaponclicker', '_system', 'location=yes');
            });
        }

        function setupSocialUnit() {
            layer.getBind('special.social.title').innerHTML = LP('special.social.title');
            layer.getBind('special.social').OnTouchClick(function() {
                cordova.InAppBrowser.open('https://vk.com/public104689708', '_system', 'location=yes');
            });
        }

        function setupOfferUnit() {
            layer.getBind('special.offer.category').innerHTML = LP('special.offer.category');
            layer.getBind('special.offer.play').innerHTML = LP('special.offer.play');
            layer.getBind('special.offer').OnTouchClick(function() {
                cordova.InAppBrowser.open('https://play.google.com/store/apps/details?id=com.Company.ValoriRos.KingsClicker', '_system', 'location=yes');
                IAds.showInterstitial();
            });
        }

        function setupLanguageUnit() {
            layer.getBind('special.language.title').innerHTML = LP('special.language.title');

            var lRu = layer.getBind('special.language.ru');
            var lEn = layer.getBind('special.language.en');

            lRu.innerHTML = 'русский';
            lEn.innerHTML = 'English';

            lRu.OnTouchClick(eventChangeLanguagePack.bind(null, 'ru'));
            lEn.OnTouchClick(eventChangeLanguagePack.bind(null, 'en'));

            updateLanguageUI();
            function updateLanguageUI() {
                switch(Wpcl.Package.Language.languageID) {
                    case 'ru':
                        lRu.classList.add('special_lang_active');
                        lEn.classList.remove('special_lang_active');
                        break;
                    case 'en':
                        lRu.classList.remove('special_lang_active');
                        lEn.classList.add('special_lang_active');
                        break;
                }
            }

            function eventChangeLanguagePack(id) {
                switch(id) {
                    case 'ru':
                        alert("Изменения вступят в силу после перезагрузки игры");
                        break;
                    case 'en':
                        alert("changes will take effect after you restart the game");
                        break;
                }
                Wpcl.Package.Language.languageID = id;
                updateLanguageUI();
            }
        }

        /*function setupPolandFriendAdUnit() {
            var startTime = 1466248292955;
            var lifeTime = 1000 * 60 * 60 * 24 * 14;
            var d = startTime + lifeTime;
            if(d > Date.now()) {
                if(layer.getBind("special.poland.youtube") && layer.getBind("special.poland.youtube").parentElement)
                    layer.getBind("special.poland.youtube").parentElement.removeChild(layer.getBind("special.poland.youtube"));
            } else {
                layer.getBind("special.poland.youtube").OnTouchClick(function() {
                    cordova.InAppBrowser.open('', '_system', 'location=yes');
                });
            }
        }*/

    });
    Wpcl.Windows.Special.prototype = Object.create(NovaEngine.HtmlDocument.HtmlWindow.prototype);
    Wpcl.Windows.Special.prototype.constructor = NovaEngine.HtmlDocument.HtmlWindow;
})();