NovaEngine.InstantiateFunction("System.Storage", function DictionaryDynamic(storageID, primaryKey)
{
    // Хранилище
    var storage = new NovaEngine.System.Storage.LocalStorageItem(storageID, LOCAL_STORAGE_ITEM_VALUE_TYPE.OBJECT, true);

    this.__defineGetter__("_data", function() { return storage.value; });

    // Возвращает данные из хранилища
    this.getStorageData = function()
    {
        var data = storage.value;
        return Object.prototype.isPrototypeOf(data) ? data : {};
    };
    
    // {key:value, key:value}
    this.get = function (tableID, parameters)
    {
        if(!tableID || (typeof tableID != "string"))
            return console.error("Arg tableID is missing");

        if(!Object.prototype.isPrototypeOf(parameters) || Array.isArray(parameters))
            return console.error("Arg parameters must be an object");

        // Загружаем уже существующий список
        var list = storage.value;
        list = (!Object.prototype.isPrototypeOf(list) || Array.isArray(list)) ? {} : list;

        // Таблица
        var table = list[tableID];
        if(!table) return undefined;

        // Коллекции таблицы
        var collections = table;
        if(!collections) return undefined;

        var pkey = parameters[primaryKey];

        // Коллекция в случае указанного первичного ключа
        var collection = collections[pkey];
        if(!!pkey && !collection) return undefined;

        delete parameters[primaryKey];

        // В параметрах поиска был указан только первичный ключ
        if(!Object.keys(parameters).length) return CollectRow(pkey, collection);

        // Сравнение параметров поиска в коллекции с указанным первичным ключом
        else if(!!collection)
        {
            for(var key in parameters)
                if(collection[key] != parameters[key]) return undefined;

            // Все ключи сопадают
            return CollectRow(pkey, collection);
        }

        // Коллекции удовлетворяющие поиску
        var f_collections = {};

        for(var key in parameters)
        {
            if(typeof parameters[key] != "string" && typeof parameters[key] != "number") continue;

            // Обход по коллекциям
            for(var c in collections)
            {
                // Обход параметров коллекции
                for(var cc in collections[c])
                {
                    if(cc == key)
                    {
                        if(collections[c][cc] == parameters[key])
                        {
                            f_collections[c] = collections[c];
                        }
                        else
                        {
                            delete f_collections[c];
                        }
                    }
                }
            }
        }

        var sfc = [];
        for(var c in f_collections) sfc.push(CollectRow(c, f_collections[c]));

        return sfc;
    };

    this.getAllCollections = function (tableID)
    {

        // Загружаем уже существующий список
        var list = storage.value;
        if(!Object.prototype.isPrototypeOf(list) || Array.isArray(list)) list = {};

        var collections = [];

        for(var c in list[tableID])
            collections.push(CollectRow(c, list[tableID][c]));

        return collections;
    };

    //
    this.remove = function(tableID, key) {

        // Загружаем уже существующий список
        var list = storage.value;

        if(!list || !list[tableID]) return;
        delete list[tableID][key];

        if(!Object.keys(list[tableID]).length)
            delete list[tableID];

        // Сохраняем
        storage.value = list;
    };

    //
    this.clear = function ()
    {
        storage.value = {};
    };

    this.insert = function()
    {
        var transaction = new Transaction();
        transaction.insert(arguments[0], arguments[1]);
        transaction.commit();
    };

    this.transaction = function() { return new Transaction(); };

    function CollectRow(key, collection)
    {
        collection[primaryKey] = key;
        return collection;
    }

    function Transaction()
    {
        var list = storage.value;
        if(!Object.prototype.isPrototypeOf(list) || Array.isArray(list)) list = {};

        // Вставляет ряд
        this.insert = function(tableID, data)
        {
            if(!tableID || (typeof tableID != "string"))
                return console.error("Arg tableID is missing");

            // Итем обязательно должен быть объектом
            if(!Object.prototype.isPrototypeOf(data) || Array.isArray(data))
                return console.error("An item must be an object");

            // В итеми обязательно должен присутствовать первичный ключ
            if(!data[primaryKey] || (typeof data[primaryKey] != "string"))
                return console.error("The primary key is missing");

            // Первичный ключ
            var key = data[primaryKey];

            // Удаляем первичный ключ
            delete data[primaryKey];

            // Записывае итем в лист
            list[tableID] = list[tableID] || {}; // Таблица
            list[tableID][key] = data;
        };

        // Операция применения
        this.commit = function()
        {
            storage.value = list;
            list = {};
        };
    }
});