var LOCAL_STORAGE_ITEM_VALUE_TYPE = {};
LOCAL_STORAGE_ITEM_VALUE_TYPE.FLOAT = 0x0;
LOCAL_STORAGE_ITEM_VALUE_TYPE.INT = 0x1;
LOCAL_STORAGE_ITEM_VALUE_TYPE.STRING = 0x2;
LOCAL_STORAGE_ITEM_VALUE_TYPE.OBJECT = 0x3;
LOCAL_STORAGE_ITEM_VALUE_TYPE.BOOL = 0x4;

NovaEngine.InstantiateClass("System.Storage", function LocalStorageItem(name, valueType, encrypt)
{
    var storageItemPrefix = "com.lsi.";
    return function LocalStorageItem(name, valueType, encrypt)
    {
        var value;

        encrypt = !!encrypt;
        name = storageItemPrefix + name;

        //   
        P_NormalizeValueType(valueType);

        //
        P_LoadStorageData();

        this.__defineGetter__("value", function() {
            if(value == null) value = P_LoadStorageData();
            return value;
        });

        this.__defineSetter__("value", function(v) {
            P_SaveStorageData(v);
        });

        function P_LoadStorageData()
        {
            var v = localStorage.getItem(name);

            try
            {
                //    ,  
                if(encrypt) v = NovaEngine.System.Helpers.Crypto.decode(v);
            }
            catch(e) { NovaEngine.Debug.Log("LocalStorageItem", "Load Method. Can't decode value"); }

            switch(valueType) {
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.INT:
                    return parseInt(v) || 0;
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.FLOAT:
                    return parseFloat(v) || 0;
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.STRING:
                    return v;
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.BOOL:
                    return (v === "true");
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.OBJECT:
                    try { return JSON.parse(v) || {}; }
                    catch(e) { return {}; } break;
                default: return v;
            }
        }

        function P_SaveStorageData(v)
        {
            switch(valueType)
            {
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.INT:
                    v = parseInt(v);
                    break;
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.FLOAT:
                    v = parseFloat(v);
                    break;
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.BOOL:
                    v = (v == true) ? "true" : "false";
                    break;
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.STRING: break;
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.OBJECT:
                    try { v = JSON.stringify(v); }
                    catch(e) { v = {}; } break;
                default: break;
            }

            try { v = encrypt ? NovaEngine.System.Helpers.Crypto.encode(v) : v; }
            catch(e) {}

            localStorage.setItem(name, v);
            value = P_LoadStorageData();
        }

        //   ,   
        function P_NormalizeValueType()
        {
            switch (valueType)
            {
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.FLOAT:
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.INT:
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.STRING:
                case LOCAL_STORAGE_ITEM_VALUE_TYPE.OBJECT:
                    return valueType;
                default:
                    return LOCAL_STORAGE_ITEM_VALUE_TYPE.STRING;
            }
        }
    }
});